@@ -36,7 +36,7 @@ class Agent:
         self.client = AsyncOpenAI(api_key=openai_key)
 
         self.tools = tools if tools else []
-        self.tools_schema = [{"type": "function", "function": tool.to_schema} for tool in self.tools]
+        self.tools_schema = [tool.to_schema for tool in self.tools]
 
         self.log.debug(f"Created gpt like tools -> {self.tools_schema}")
 
@@ -51,7 +51,16 @@ class Agent:
         await self.db.ex(dmth.UpdateOne(Chat, chat, to_update="status"))
 
         messages: list[Message] = await self.db.ex(dmth.GetMany(Message, chat_id=chat.id))
-        gpt_messages = list(map(lambda x: x.gpt_dump, messages))
+        gpt_messages = []
+        for message in messages:
+            data = message.gpt_dump
+
+            if isinstance(data, list):
+                for row in data:
+                    gpt_messages.append(row)
+
+            else:
+                gpt_messages.append(data)
 
         tools_schema = self.tools_schema[:]
 
@@ -82,18 +91,19 @@ class Agent:
 
             raise err
 
-        response_types = []
+        responses = []
         for i in response.output:
             if i.__class__ not in [ResponseFunctionToolCall, ResponseOutputMessage]:
                 self.log.warning(f"Unsupported response type -> {i.__class__}")
                 continue
 
-            response_types.append(i.__class__)
+            responses.append(i)
 
-        if len(response_types) != 1:
-            raise TypeError(f"Got many response types in 1 output -> {response_types}")
+        if len(responses) != 1:
+            self.log.debug(json.dumps(response.model_dump(), indent=2, ensure_ascii=False))
+            raise TypeError(f"Got many response types in 1 output -> {responses}")
 
-        if isinstance(response_types[0], ResponseFunctionToolCall):
+        if isinstance(responses[0], ResponseFunctionToolCall):
             return AgentResponse(
                 type=AgentResponseType.tools,
                 content=[
@@ -102,32 +112,29 @@ class Agent:
                         name=i.name,
                         arguments=i.arguments
                     )
-                    for i in response.output
+                    for i in responses
                 ]
             )
 
-        elif isinstance(response_types[0], ResponseOutputMessage):
-            content_types = []
-            for i in response.output:
-                if i.content.__class__ in content_types:
-                    continue
+        elif isinstance(responses[0], ResponseOutputMessage):
+            texts = []
+            for i in responses:
+                for j in i.content:
+                    if j.__class__ not in [ResponseOutputText]:
+                        self.log.warning(f"Unsupported content type -> {j.__class__}")
+                        continue
 
-                content_types.append(i.content.__class__)
-
-            if len(content_types) != 1:
-                self.log.warning(f"Got many content types in 1 output -> {content_types}")
-
-            if not isinstance(content_types[0], ResponseOutputText):
-                self.log.warning(f"Unsupported content type -> {content_types[0]}")
+                    texts.append(j.text)
 
             return AgentResponse(
                 type=AgentResponseType.answer,
-                content="\n".join([
-                    i.content.text
-                    for i in response.output if isinstance(i.content, ResponseOutputText)
-                ])
+                content="\n\n".join(texts)
             )
 
+        else:
+            self.log.debug(json.dumps(response.model_dump(), indent=2, ensure_ascii=False))
+            raise TypeError(f"Unsupported response type -> {responses[0]}")
+
     async def generate(self, chat: Chat) -> AgentResponse:
         try:
             return await self.__generate(chat)
