@@ -1,4 +1,4 @@
-
+import asyncio
 import functools
 import typing
 import aiohttp
@@ -7,6 +7,7 @@ from loguru import logger
 
 from .event import ToolEvent
 
+from AgentService.enums.connector import ToolResponse
 from AgentService.types.response import AgentResponse, AgentResponseType
 from AgentService.dtypes.message import SendMessageRequest, Tool, ToolAnswer
 
@@ -122,8 +123,13 @@ class AgentConnector:
                 text="function not binded"
             )
 
-        self.log.debug(f"Dispatching event -> {event} -> {callback}")
-        return await callback(event=event)
+        response = ToolResponse.retry
+        while response == ToolResponse.retry:
+            response = await callback(event=event)
+            self.log.debug(f"Dispatching event -> {event} -> {callback} -> {response}")
+            await asyncio.sleep(5)
+
+        return response
 
     async def handle_tools(self, tools: list[Tool], chat_id) -> list[ToolAnswer]:
         tool_answers = [
